# Generated by Django 5.0.1 on 2024-02-04 22:24

import django.db.models.deletion
from django.conf import settings
from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="PasswordRecord",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "password",
                    models.CharField(editable=False, max_length=128, verbose_name="Password hash"),
                ),
                ("date", models.DateTimeField(auto_now_add=True, verbose_name="Date")),
                (
                    "user",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="password_records",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "ordering": ["-date"],
                "get_latest_by": "date",
            },
        ),
    ]



# Generated by Django 5.0.1 on 2024-02-04 22:24


from django.db import migrations


def create_password_record(apps, schema_editor):
    # We get the model from the versioned app registry;
    # if we directly import it, it'll be the wrong version
    User = apps.get_model("auth", "User")
    PasswordRecord = apps.get_model("better_passwords", "PasswordRecord")

    for user in User.objects.all():
        PasswordRecord.objects.create(user=user, password=user.password)


class Migration(migrations.Migration):
    dependencies = [
        ("better_passwords", "0001_initial"),
    ]

    operations = [migrations.RunPython(create_password_record)]



from django.conf import settings
from django.contrib import messages
from django.shortcuts import redirect
from django.urls import (
    resolve,
    reverse,
)
from django.utils import timezone
from django.utils.translation import gettext as _


class PasswordExpirationMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response
        self.expiration_days = timezone.timedelta(
            days=getattr(settings, "DBP_PASSWORD_EXPIRATION_DAYS", 60)
        )

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.

        resolver_match = resolve(request.path)
        user = request.user

        if user.is_authenticated:
            latest_record = user.password_records.latest()
            records_count = user.password_records.count()

            if ((timezone.now() - latest_record.date) >= self.expiration_days) or (
                records_count <= 1
            ):
                if resolver_match.app_name == "admin" and resolver_match.url_name not in (
                    "password_change",
                    "logout",
                ):
                    return redirect(
                        reverse(
                            "admin:password_change",
                            current_app=resolver_match.namespace,
                        )
                    )
                elif (
                    hasattr(settings, "DBP_PASSWORD_CHANGE_REDIRECT_URL")
                    and bool(settings.DBP_PASSWORD_CHANGE_REDIRECT_URL)
                    and resolver_match.url_name
                    not in (
                        settings.DBP_PASSWORD_CHANGE_REDIRECT_URL,
                        settings.DBP_LOGOUT_URL,
                    )
                ):
                    return redirect(
                        reverse(
                            settings.DBP_PASSWORD_CHANGE_REDIRECT_URL,
                            current_app=resolver_match.namespace,
                        )
                    )

                if request.method == "GET":
                    message = (
                        "Update your password to get started."
                        if records_count <= 1
                        else f"Your password is at least {self.expiration_days.days} days old and needs to be updated."
                    )

                    messages.add_message(
                        request,
                        messages.WARNING,
                        _(message),
                        fail_silently=True,
                    )

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        return response



from django.conf import settings
from django.db import models


class PasswordRecord(models.Model):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="password_records",
        on_delete=models.CASCADE,
        editable=False,
    )

    # max_length django.contrib.auth.base_user.AbstractBaseUser.password
    password = models.CharField("Password hash", max_length=128, editable=False)
    date = models.DateTimeField("Date", auto_now_add=True, editable=False)

    class Meta:
        get_latest_by = "date"
        ordering = ["-date"]



from django.core.exceptions import ValidationError
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _


class CustomPasswordValidator:
    """
    - Validates password maximum and minimum length;
    - Ensures that it contains required characters, at least one digit, one
    upper case and one lower case letters;
    - Ensures that it not contains forbidden characters;

    Args:
        - min_length (Optional[int]): minimum length, defaults to 6
        - max_length (Optional[int]): maximum length, defaults to 30
        - required_characters (Optional[str]): characters that must be present,
        defaults to ''
        - forbidden_characters (Optional[str]): characters that cannot be present,
        defaults to ''
    """

    def __init__(
        self,
        min_length: int = 6,
        max_length: int = 30,
        required_characters: str = "",
        forbidden_characters: str = "",
    ):
        self.min_length = min_length
        self.max_length = max_length
        self.required_characters = required_characters
        self.forbidden_characters = forbidden_characters

    def validate(self, password, user=None):
        self._validate_min_length(password=password)
        self._validate_max_length(password=password)
        self._validate_required_characters(password=password)
        self._validate_forbidden_characters(password=password)
        self._validate_digits(password=password)
        self._validate_upper_case_letter(password=password)
        self._validate_lower_case_letter(password=password)

    def _validate_min_length(self, password: str):
        if len(password) < self.min_length:
            raise ValidationError(
                _(f"Password must contain at least {self.min_length} characters.")
            )

    def _validate_max_length(self, password: str):
        if len(password) > self.max_length:
            raise ValidationError(
                _(f"Password cannot be longer than {self.max_length} characters.")
            )

    def _validate_required_characters(self, password: str):
        if self.required_characters and not any(
            char in password for char in self.required_characters
        ):
            raise ValidationError(
                _(
                    f"Password must contain at least one of the following characters: {self.required_characters}"
                )
            )

    def _validate_forbidden_characters(self, password: str):
        if self.forbidden_characters and any(
            char in self.forbidden_characters for char in password
        ):
            raise ValidationError(
                _(f"Password cannot contain the following characters: {self.forbidden_characters}.")
            )

    def _validate_digits(self, password: str):
        if not any(char.isdigit() for char in password):
            raise ValidationError(_("Password must contain at least one digit."))

    def _validate_upper_case_letter(self, password: str):
        if not any(char.isupper() for char in password):
            raise ValidationError(_("Password must contain at least one upper case letter."))

    def _validate_lower_case_letter(self, password: str):
        if not any(char.islower() for char in password):
            raise ValidationError(_("Password must contain at least one lower case letter."))

    def get_help_text(self):
        help_text = (
            f"Your password must be between {self.min_length} and {self.max_length} characters lenght.<br>"
            "Your password must contain at least one digit, one upper case and one lower case letters.<br>"
        )

        if self.required_characters:
            help_text += f"Your password must contain at least one of the following characters: {self.required_characters}<br>"

        if self.forbidden_characters:
            help_text += f"Your password cannot contain the following characters: {self.forbidden_characters}.<br>"

        return _(mark_safe(help_text))
